##
# File:    ValidationUtils.py
# Date:    4-Sep-2012
#
# Updates:
#  4-Sep-2012  jdw add validation report method
#  13-Dec-2012 jdw added method for version 2 of validation module
#  15-Jan-2014 jdw add annotation context user paramter to validation report
#  24-Jan-2014 jdw add addition report output options
#  21-Sep-2014 jdw update validationReportTestOp() for NMR
#  16-Jan-2015 jdw update validationReportTestOp() to test for experimental files -
#  22-Mar-2015 jdw add entry_id as parameter in validationReportAltOp
#  16-Sep-2015 jdw add validationReportAllOp
#
#  14-Jul-2016 jdw validationReportAllOp() is the only valid entry point here - all others are deprecated.
#  14-Jul-2016 jdw add 'validation_mode' input parameter -
#  18-Dec-2016 ep  remove unused validationReportAltOp and validationReportV2Op
#  10-May-2019 ep  add validationReportAllOpV2 for export of edmap coefficients
#  30-Jun-2020 zf  modify to be compatible with the changes of image tar file output in RcsbDpUtility & ValidationWrapper
##
"""
Module of annotation utility operations supporting the call protocol of the ProcessRunner() class.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import os
import sys
import traceback
from wwpdb.utils.wf.plugins.UtilsBase import UtilsBase
from wwpdb.utils.config.ConfigInfo import ConfigInfo

from wwpdb.utils.dp.RcsbDpUtility import RcsbDpUtility
from wwpdb.utils.dp.ValidationWrapper import ValidationWrapper

# For remediation of legacy CS files in annotation.
# Not a requirements for wwpdb.utils.wf - but if running validation, it will have pulled in wwpdb.utils.nmr
# into the virtual environment
try:
    from wwpdb.utils.nmr.CifToNmrStar import CifToNmrStar
    from mmcif.io.IoAdapterPy import IoAdapterPy
    import tempfile
except ImportError:
    pass


def remediate_cs_file(infile, outfile):
    """Produces an NMR* formatted from input CIF.  Correcting missing section headers that are required"""
    ctns = CifToNmrStar()
    return ctns.convert(cifPath=infile, strPath=outfile)


def starToPdbx(starPath=None, pdbxPath=None, log=sys.stderr):
    """Converts NMR* to mmCIF"""
    if starPath is None or pdbxPath is None:
        return False
    #
    try:
        myIo = IoAdapterPy(False, sys.stderr)
        containerList = myIo.readFile(starPath)
        if containerList is not None and len(containerList) > 1:
            log.write("Input container list is  %r\n" % ([(c.getName(), c.getType()) for c in containerList]))
            for c in containerList:
                c.setType("data")
            # myIo.writeFile(pdbxPath, containerList=containerList[1:])
            myIo.writeFile(pdbxPath, containerList=containerList)
            return True
    except Exception as _e:  # noqa: F841
        traceback.print_exc(file=log)

    return False


class ValidationUtils(UtilsBase):

    """Utility class to run validation operations.

    Current supported operations include:

    - create PDF report and supporting XML data file

    Each method in this class implements the method calling interface of the
    `ProcessRunner()` class.   This interface provides the keyword arguments:

    - inputObjectD   dictionary of input objects
    - outputObjectD  dictionary of output objects
    - userParameterD  dictionary of user adjustable parameters
    - internalParameterD dictionary of internal parameters

    Each method in the class handles its own exceptions and returns
    True on success or False otherwise.

    """

    def __init__(self, verbose=False, log=sys.stderr):
        super(ValidationUtils, self).__init__(verbose, log)
        self.__cleanUp = False
        """Flag to remove any temporary directories created by this class.
        """

    def validationReportAllOp(self, **kwArgs):
        """Create validation reports, supporting XML data, and image files for the input PDBx model file and
        the current implementation of all experimental methods  with optional input
        of experimental data files --

        """
        try:
            (inpObjD, outObjD, uD, _pD) = self._getArgs(kwArgs)
            pdbxPath = inpObjD["src1"].getFilePathReference()
            depDataSetId = inpObjD["src1"].getDepositionDataSetId()
            #
            if "src2" in inpObjD:
                sfPath = inpObjD["src2"].getFilePathReference()
            else:
                sfPath = None

            if "src3" in inpObjD:
                csPath = inpObjD["src3"].getFilePathReference()
            else:
                csPath = None

            if "src4" in inpObjD:
                volPath = inpObjD["src4"].getFilePathReference()
            else:
                volPath = None

            if "src5" in inpObjD:
                fscPath = inpObjD["src5"].getFilePathReference()
            else:
                fscPath = None

            validationReportPath = outObjD["dst1"].getFilePathReference()
            xmlReportPath = outObjD["dst2"].getFilePathReference()

            validationFullReportPath = outObjD["dst3"].getFilePathReference()
            pngReportPath = outObjD["dst4"].getFilePathReference()
            svgReportPath = outObjD["dst5"].getFilePathReference()
            cifReportPath = outObjD["dst6"].getFilePathReference()

            dirPath = outObjD["dst1"].getDirPathReference()
            imageTarPath = os.path.join(dirPath, "val-report-images.tar")
            logPath = os.path.join(dirPath, "val-report-all.log")
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            dp = RcsbDpUtility(tmpPath=dirPath, siteId=siteId, verbose=self._verbose, log=self._lfh)
            dp.imp(pdbxPath)
            #
            validationMode = str(uD["validation_mode"])
            if validationMode in ["annotate", "deposit", "server", "release", "legacy"]:
                dp.addInput(name="request_validation_mode", value=validationMode)

            #  add an control over the report context
            inAnnotation = str(uD["in_annotation"])
            if inAnnotation in ["yes", "no"]:
                dp.addInput(name="request_annotation_context", value=inAnnotation)
            #
            # add parameters only if these exist --
            #
            if depDataSetId is not None and len(depDataSetId) > 3:
                dp.addInput(name="entry_id", value=depDataSetId)

            if sfPath is not None and os.access(sfPath, os.R_OK):
                dp.addInput(name="sf_file_path", value=sfPath)
            else:
                sfPath = None
            #
            if csPath is not None and os.access(csPath, os.R_OK):
                dp.addInput(name="cs_file_path", value=csPath)
            else:
                csPath = None

            if volPath is not None and os.access(volPath, os.R_OK):
                dp.addInput(name="vol_file_path", value=volPath)
            else:
                volPath = None

            if fscPath is not None and os.access(fscPath, os.R_OK):
                dp.addInput(name="fsc_file_path", value=fscPath)
            else:
                fscPath = None

            dp.op("annot-wwpdb-validate-all")
            dp.expLog(logPath)
            dp.expList(dstPathList=[validationReportPath, xmlReportPath, validationFullReportPath, pngReportPath, svgReportPath, imageTarPath, cifReportPath])

            if self._verbose:
                self._lfh.write("+ValidationUtils.validationReportAllOp() - Entry Id:                %s\n" % depDataSetId)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - PDBx   file path:        %s\n" % pdbxPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - SF     file path:        %s\n" % sfPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - CS     file path:        %s\n" % csPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - Volume file path:        %s\n" % volPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - Fsc    file path:        %s\n" % fscPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - PDF report file path:  %s\n" % validationReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - XML report file path:  %s\n" % xmlReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - CIF report file path:  %s\n" % cifReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - PNG slider file path:  %s\n" % pngReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - SVG slider file path:  %s\n" % svgReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - full PDF   file path:  %s\n" % validationFullReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOp() - validation mode:       %s\n" % validationMode)

            if self.__cleanUp:
                dp.cleanup()
            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False
        #

    def validationReportAllOpV2(self, **kwArgs):
        """Create validation reports, supporting XML data, and image files for the input PDBx model file and
        the current implementation of all experimental methods  with optional input
        of experimental data files --

        Supports output of 2fo and fo edmap coefficients
        """
        try:
            (inpObjD, outObjD, uD, _pD) = self._getArgs(kwArgs)
            pdbxPath = inpObjD["src1"].getFilePathReference()
            depDataSetId = inpObjD["src1"].getDepositionDataSetId()
            #
            if "src2" in inpObjD:
                sfPath = inpObjD["src2"].getFilePathReference()
            else:
                sfPath = None

            if "src3" in inpObjD:
                csPath = inpObjD["src3"].getFilePathReference()
            else:
                csPath = None

            if "src4" in inpObjD:
                volPath = inpObjD["src4"].getFilePathReference()
            else:
                volPath = None

            if "src5" in inpObjD:
                authorFSCPath = inpObjD["src5"].getFilePathReference()
            else:
                authorFSCPath = None

            if "src6" in inpObjD:
                emdbXMLPath = inpObjD["src6"].getFilePathReference()
            else:
                emdbXMLPath = None

            validationReportPath = outObjD["dst1"].getFilePathReference()
            xmlReportPath = outObjD["dst2"].getFilePathReference()

            validationFullReportPath = outObjD["dst3"].getFilePathReference()
            pngReportPath = outObjD["dst4"].getFilePathReference()
            svgReportPath = outObjD["dst5"].getFilePathReference()
            coeffoReportPath = outObjD["dst6"].getFilePathReference()
            coef2foReportPath = outObjD["dst7"].getFilePathReference()
            cifReportPath = outObjD["dst8"].getFilePathReference()

            dirPath = outObjD["dst1"].getDirPathReference()
            imageTarPath = os.path.join(dirPath, "val-report-images.tar")
            logPath = os.path.join(dirPath, "val-report-all.log")
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            vw = ValidationWrapper(tmpPath=dirPath, siteId=siteId, verbose=self._verbose, log=self._lfh)
            vw.imp(pdbxPath)
            #
            validationMode = str(uD["validation_mode"])
            if validationMode in ["annotate", "deposit", "server", "release", "legacy"]:
                vw.addInput(name="request_validation_mode", value=validationMode)

            #  add an control over the report context
            inAnnotation = str(uD["in_annotation"])
            if inAnnotation in ["yes", "no"]:
                vw.addInput(name="request_annotation_context", value=inAnnotation)
            #
            # Add file cleanup - if desired
            #
            alwaysCleanup = cI.get("WF_VALIDATION_REMOVE", None)
            if alwaysCleanup:
                vw.addInput(name="always_clear_calcs", value="Y")
            #
            # add parameters only if these exist --
            #
            if depDataSetId is not None and len(depDataSetId) > 3:
                vw.addInput(name="entry_id", value=depDataSetId)

            if sfPath is not None and os.access(sfPath, os.R_OK):
                vw.addInput(name="sf_file_path", value=sfPath)
            else:
                sfPath = None
            #
            if csPath is not None and os.access(csPath, os.R_OK):
                vw.addInput(name="cs_file_path", value=csPath)
            else:
                csPath = None

            if volPath is not None and os.access(volPath, os.R_OK):
                vw.addInput(name="vol_file_path", value=volPath)
            else:
                volPath = None

            if authorFSCPath is not None and os.access(authorFSCPath, os.R_OK):
                vw.addInput(name="fsc_file_path", value=authorFSCPath)
            else:
                authorFSCPath = None

            if emdbXMLPath is not None and os.access(emdbXMLPath, os.R_OK):
                vw.addInput(name="emdb_xml_path", value=emdbXMLPath)
            else:
                emdbXMLPath = None

            vw.op("annot-wwpdb-validate-all-sf")
            vw.expLog(logPath)
            vw.expList(
                dstPathList=[
                    validationReportPath,
                    xmlReportPath,
                    validationFullReportPath,
                    pngReportPath,
                    svgReportPath,
                    imageTarPath,
                    cifReportPath,
                    coeffoReportPath,
                    coef2foReportPath,
                ]
            )

            if self._verbose:
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - Entry Id:                %s\n" % depDataSetId)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - PDBx   file path:        %s\n" % pdbxPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - SF     file path:        %s\n" % sfPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - CS     file path:        %s\n" % csPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - Volume file path:        %s\n" % volPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - FSC    file path:        %s\n" % authorFSCPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - EMDB XML   file path:  %s\n" % emdbXMLPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - PDF report file path:  %s\n" % validationReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - XML report file path:  %s\n" % xmlReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - CIF report file path:  %s\n" % cifReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - PNG slider file path:  %s\n" % pngReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - SVG slider file path:  %s\n" % svgReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - full PDF   file path:  %s\n" % validationFullReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - fo coef file path:     %s\n" % coeffoReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - 2fo coef file path     %s\n" % coef2foReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV2() - validation mode:       %s\n" % validationMode)

            if self.__cleanUp:
                vw.cleanup()
            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False
        #

    def validationReportAllOpV3(self, **kwArgs):
        """Create validation reports, supporting XML data, and image files for the input PDBx model file and
        the current implementation of all experimental methods  with optional input
        of experimental data files --

        Supports output of 2fo and fo edmap coefficients

        Supports src6 - nmr-data-str file - which will override a CS file
        """
        try:
            (inpObjD, outObjD, uD, _pD) = self._getArgs(kwArgs)
            pdbxPath = inpObjD["src1"].getFilePathReference()
            depDataSetId = inpObjD["src1"].getDepositionDataSetId()
            #
            if "src2" in inpObjD:
                sfPath = inpObjD["src2"].getFilePathReference()
            else:
                sfPath = None

            if "src3" in inpObjD:
                csPath = inpObjD["src3"].getFilePathReference()
            else:
                csPath = None

            if "src4" in inpObjD:
                volPath = inpObjD["src4"].getFilePathReference()
            else:
                volPath = None

            if "src5" in inpObjD:
                authorFSCPath = inpObjD["src5"].getFilePathReference()
            else:
                authorFSCPath = None

            if "src6" in inpObjD:
                nmrDataPath = inpObjD["src6"].getFilePathReference()
            else:
                nmrDataPath = None

            validationReportPath = outObjD["dst1"].getFilePathReference()
            xmlReportPath = outObjD["dst2"].getFilePathReference()

            validationFullReportPath = outObjD["dst3"].getFilePathReference()
            pngReportPath = outObjD["dst4"].getFilePathReference()
            svgReportPath = outObjD["dst5"].getFilePathReference()
            coeffoReportPath = outObjD["dst6"].getFilePathReference()
            coef2foReportPath = outObjD["dst7"].getFilePathReference()
            cifReportPath = outObjD["dst8"].getFilePathReference()

            dirPath = outObjD["dst1"].getDirPathReference()
            imageTarPath = os.path.join(dirPath, "val-report-images.tar")
            logPath = os.path.join(dirPath, "val-report-all.log")
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            vw = ValidationWrapper(tmpPath=dirPath, siteId=siteId, verbose=self._verbose, log=self._lfh)
            vw.imp(pdbxPath)
            #
            validationMode = str(uD["validation_mode"])
            if validationMode in ["annotate", "deposit", "server", "release", "legacy"]:
                vw.addInput(name="request_validation_mode", value=validationMode)

            #  add an control over the report context
            inAnnotation = str(uD["in_annotation"])
            if inAnnotation in ["yes", "no"]:
                vw.addInput(name="request_annotation_context", value=inAnnotation)

            #
            # Add file cleanup - if desired
            #
            alwaysCleanup = cI.get("WF_VALIDATION_REMOVE", None)
            if alwaysCleanup:
                vw.addInput(name="always_clear_calcs", value="Y")

            #
            # add parameters only if these exist --
            #
            if depDataSetId is not None and len(depDataSetId) > 3:
                vw.addInput(name="entry_id", value=depDataSetId)

            if sfPath is not None and os.access(sfPath, os.R_OK):
                vw.addInput(name="sf_file_path", value=sfPath)
            else:
                sfPath = None
            #
            # If nmr-data and cs data - use nmr-data -- this happens in DepUI
            if nmrDataPath is not None and os.access(nmrDataPath, os.R_OK):
                vw.addInput(name="cs_file_path", value=nmrDataPath)
                vw.addInput(name="nmr_restraint_file_path", value=nmrDataPath)
            else:
                nmrDataPath = None

            if nmrDataPath is None and csPath is not None and os.access(csPath, os.R_OK):
                if inAnnotation in ["yes"]:
                    tempd = tempfile.TemporaryDirectory(dir=os.path.dirname(csPath))
                    self._lfh.write("Remediating CS file for validatiobn in %s\n" % tempd.name)

                    tempnmrstarPath = os.path.join(tempd.name, os.path.basename(csPath) + ".str")
                    newcsPath = os.path.join(tempd.name, os.path.basename(csPath))
                    starToPdbx(csPath, tempnmrstarPath, self._lfh)
                    remediate_cs_file(tempnmrstarPath, newcsPath)

                    csPath = newcsPath

                vw.addInput(name="cs_file_path", value=csPath)
            else:
                csPath = None

            if volPath is not None and os.access(volPath, os.R_OK):
                vw.addInput(name="vol_file_path", value=volPath)
            else:
                volPath = None

            if authorFSCPath is not None and os.access(authorFSCPath, os.R_OK):
                vw.addInput(name="fsc_file_path", value=authorFSCPath)
            else:
                authorFSCPath = None

            vw.op("annot-wwpdb-validate-all-sf")
            vw.expLog(logPath)
            vw.expList(
                dstPathList=[
                    validationReportPath,
                    xmlReportPath,
                    validationFullReportPath,
                    pngReportPath,
                    svgReportPath,
                    imageTarPath,
                    cifReportPath,
                    coeffoReportPath,
                    coef2foReportPath,
                ]
            )

            if self._verbose:
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - Entry Id:                %s\n" % depDataSetId)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - PDBx    file path:       %s\n" % pdbxPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - SF      file path:       %s\n" % sfPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - CS      file path:       %s\n" % csPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - NMRdata file path:       %s\n" % nmrDataPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - Volume  file path:       %s\n" % volPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - FSC     file path:       %s\n" % authorFSCPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - PDF report file path:  %s\n" % validationReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - XML report file path:  %s\n" % xmlReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - CIF report file path:  %s\n" % cifReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - PNG slider file path:  %s\n" % pngReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - SVG slider file path:  %s\n" % svgReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - full PDF   file path:  %s\n" % validationFullReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - fo coef file path:     %s\n" % coeffoReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - 2fo coef file path     %s\n" % coef2foReportPath)
                self._lfh.write("+ValidationUtils.validationReportAllOpV3() - validation mode:       %s\n" % validationMode)

            if self.__cleanUp:
                vw.cleanup()
            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False
        #



##
# File:    UtilsBase.py
# Date:    8-April-2010
#
# Updates:
#
##
"""
Module containing the base class describing the call interface for methods callable
from the ProcessRunner() class.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import sys


class UtilsBase(object):

    """Base class implementing the method calling interface of the
    `ProcessRunner()` class.   This interface provides the keyword arguments:

    - inputObjectD   dictionary of input objects
    - outputObjectD  dictionary of output objects
    - userParmeterD  dictionary of user adjustable parameters
    - internalParameterD dictionary of internal parameters

    Each method in the class handles its own exceptions and returns
    True on success or False otherwise.

    """

    def __init__(self, verbose=True, log=sys.stderr):
        self._verbose = verbose
        self._lfh = log

    def _getArgs(self, kwD):
        """Extract the keyword arguments used by methods in this class."""
        return (kwD["inputObjectD"], kwD["outputObjectD"], kwD["userParameterD"], kwD["internalParameterD"])

    def noOp(self, **kwArgs):
        self._getArgs(kwArgs)
        return True

    def dumpArgs(self, kwD):
        for k, v in kwD.items():
            self._lfh.write("+UtilsBase.dumArgs() key: %s  value: %r \n" % (k, v))



##
# File:    PdxUtils.py
# Date:    10-April-2010
#
# Updates:
# 24-April-2010  jdw statusOp method to return a dictionary of common status items.
#
##
"""
Module of data access utility operations supporting the call protocol of the ProcessRunner() class.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import sys
import traceback

from wwpdb.utils.wf.plugins.UtilsBase import UtilsBase

from mmcif.core.mmciflib import ParseCifSimple  # pylint: disable=no-name-in-module


class CifFile(object):

    """
    CifFile
    """

    def __init__(self, fileName):
        self.__fileName = fileName
        self.__cifFile = ParseCifSimple(self.__fileName, verbose=False, intCaseSense=0, maxLineLength=1024, nullValue="?", parseLogFileName="")

    def getCifFile(self):
        return self.__cifFile

    @classmethod
    def getFileExt(cls):
        return "cif"

    def write(self, fileName):
        self.__cifFile.Write(fileName)

    @classmethod
    def read(cls, fileName):
        return cls(fileName)


class PdbxUtils(UtilsBase):

    """Utility class of methods to access data within PDBx files.

    Current supported operations include:

    Each method in this class implements the method calling interface of the
    `ProcessRunner()` class.   This interface provides the keyword arguments:

    - inputObjectD   dictionary of input objects
    - outputObjectD  dictionary of output objects
    - userParameterD  dictionary of user adjustable parameters
    - internalParameterD dictionary of internal parameters

    Each method in the class handles its own exceptions and returns
    True on success or False otherwise.

    """

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxUtils, self).__init__(verbose, log)
        #
        self.__block = None
        """ Target block -
        """
        self.__blockList = []
        self.__targetBlockName = None
        self.__targetBlockIndex = 0
        self.__cifFile = None
        self.__blockList = []
        self._lfh = log

    def __getBlock(self, pdbxPath):
        """Open the input PDBx file and set the target data block.

        Returns:

        True for success or False otherwise.

        """
        try:
            cf = CifFile(pdbxPath)
            self.__cifFile = cf.getCifFile()
            bL = []
            self.__blockList = self.__cifFile.GetBlockNames(bL)
            # print self.__blockList

            # take the target block by name if specified or just the first block otherwise
            if self.__targetBlockName is not None and self.__targetBlockName in self.__blockList:
                self.__block = self.__cifFile.GetBlock(self.__targetBlockName)
            else:
                self.__block = self.__cifFile.GetBlock(self.__blockList[self.__targetBlockIndex])
            #
            return True
        except Exception as _e:  # noqa: F841
            if self._verbose:
                traceback.print_exc(file=self._lfh)
            return False

    def __templateSelection(self, kwD):
        """Template selection method.

        This method supports equality selection conditions within the target category.

        Selector defines the following selector parameters:
        - targetCategoryName  name of category to which selection is applied
        - targetAttributeNameList [attributeName, attributeName,...] the values of the
          attributes in this list will be returned subject to the selection condition.

        Selection Condition -
        - conditionList  [(attributeName,attributeValue,comparisonOp),,...]

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwD)
            pdbxPath = str(inpObjD["src"].getFilePathReference())
            if not self.__getBlock(pdbxPath):
                return False
            #

            targetCategory = str(inpObjD["src"].getSelectCategoryName())
            if not self.__block.IsTablePresent(targetCategory):
                return False

            myTable = self.__block.GetTable(targetCategory)

            conditionList = inpObjD["src"].getSelectConditionList()
            if len(conditionList) > 0:
                aNL = []
                aVL = []
                for cTup in conditionList:
                    aNL.append(str(cTup[0]))
                    aVL.append(str(cTup[1]))
                aNT = tuple(aNL)
                aVT = tuple(aVL)
                #
                # print "aNt",aNT
                # print "aVt",aVT
                indices = myTable.Search(aVT, aNT)
                if len(indices) > 0:
                    attributeList = inpObjD["src"].getSelectAttributeList()
                    rV = []
                    for atN in attributeList:
                        rV.append(myTable(indices[0], str(atN)))
                    outObjD["dst"].setValue(rV)
                    return True
        except Exception as _e:  # noqa: F841
            if self._verbose:
                traceback.print_exc(file=self._lfh)
            return False

        return False

    def __templateFetchAttribute(self, kwD):
        """Template fetch column method.

        This method supports recovering the values of an attribute (or column of attributes).

        Selector defines the following selector parameters:
        - targetCategoryName  name of the target category
        - targetAttributeName name of the target attribute

        Only container types *list* and *value* are support.
        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwD)
            pdbxPath = str(inpObjD["src"].getFilePathReference())
            if not self.__getBlock(pdbxPath):
                return False
            #
            targetCategory = str(inpObjD["src"].getSelectCategoryName())
            if not self.__block.IsTablePresent(targetCategory):
                return False

            attributeList = inpObjD["src"].getSelectAttributeList()
            targetAttribute = str(attributeList[0])

            myTable = self.__block.GetTable(targetCategory)
            cL = []
            # colNames = list(myTable.GetColumnNames())
            rList = list(myTable.GetColumn(cL, targetAttribute))

            if outObjD["dst"].getContainerTypeName() == "value":
                outObjD["dst"].setValue(rList[0])
            elif outObjD["dst"].getContainerTypeName() == "list":
                outObjD["dst"].setValue(rList)
            else:
                return False
            #
            return True
        except Exception as _e:  # noqa: F841
            if self._verbose:
                traceback.print_exc(file=self._lfh)
            return False

    def __getCategoryRowCount(self, kwD):
        """Template method to count category size/test for existence .

        Input src defines pdbx file and dst will hold integer row count.

        User defined parameter 'category':
        - name of category to which the test is applied


        """
        try:
            (inpObjD, outObjD, uD, _pD) = self._getArgs(kwD)
            pdbxPath = str(inpObjD["src"].getFilePathReference())
            if not self.__getBlock(pdbxPath):
                outObjD["dst"].setValue(0)
                return True
            #
            if "category" in uD:
                targetCategory = str(uD["category"])
            else:
                outObjD["dst"].setValue(0)
                return True

            if not self.__block.IsTablePresent(targetCategory):
                outObjD["dst"].setValue(0)
                return True

            myTable = self.__block.GetTable(targetCategory)
            outObjD["dst"].setValue(myTable.GetNumRows())

            return True
        except Exception as _e:  # noqa: F841
            if self._verbose:
                traceback.print_exc(file=self._lfh)
            return False

        return False

    def categoryRowCountOp(self, **kwArgs):
        return self.__getCategoryRowCount(kwArgs)

    def selectOp(self, **kwArgs):
        return self.__templateSelection(kwArgs)

    def fetchOp(self, **kwArgs):
        return self.__templateFetchAttribute(kwArgs)

    def statusOp(self, **kwArgs):
        return self.__fetchStatusDictionary(kwArgs)

    def __fetchStatusDictionary(self, kwD):
        """
        depDB = {}
        Done -
        depDB["PDB_ID"]=self.accession
        depDB["INITIAL_DEPOSITION_DATE"] = datetime.date.today()
        depDB["DEPOSIT_SITE"]="PDBe"
        depDB["PROCESS_SITE"]="PDBe"
        depDB["STATUS_CODE"]=self.processState
        depDB["ANNOTATOR_INITIALS"]="AN"
        depDB["AUTHOR_RELEASE_STATUS_CODE"]="HPUB"
        depDB["EXP_METHOD"]="X-RAY"
        depDB["STATUS_CODE_EXP"]="comp"
        depDB["TITLE"]="A test deposition for " + self.depositionID
        #
        Todo -
        depDB["SG_CENTER"]=None
        depDB["AUTHOR_LIST"] = "A.Depositor, Anna.Depositor, Yeta,Depositor"

        """
        d = {}
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwD)
            pdbxPath = inpObjD["src"].getFilePathReference()
            if not self.__getBlock(pdbxPath):
                return False
            #
            targetCategory = "struct"
            if self.__block.IsTablePresent(targetCategory):
                targetAttribute = "title"
                myTable = self.__block.GetTable(targetCategory)
                colNames = list(myTable.GetColumnNames())
                cL = []
                rList = list(myTable.GetColumn(cL, targetAttribute))
                if len(rList) > 0:
                    d["title"] = rList[0]
                else:
                    d["title"] = None
            #
            targetCategory = "exptl"
            if self.__block.IsTablePresent(targetCategory):
                targetAttribute = "method"
                myTable = self.__block.GetTable(targetCategory)
                colNames = list(myTable.GetColumnNames())
                cL = []
                rList = list(myTable.GetColumn(cL, targetAttribute))
                if len(rList) > 0:
                    d["exp_method"] = rList[0]
                else:
                    d["exp_method"] = None

            targetCategory = "pdbx_database_status"
            if self.__block.IsTablePresent(targetCategory):
                aList = [
                    ("recvd_initial_deposition_date", "recvd_initial_deposition_date"),
                    ("author_release_status_code", "author_release_status_code"),
                    ("deposit_site", "deposit_site"),
                    ("process_site", "process_site"),
                    ("status_code", "status_code"),
                    ("annotator_initials", "rcsb_annotator"),
                    ("status_code_mr", "status_code_mr"),
                    ("status_code_sf", "status_code_sf"),
                ]
                myTable = self.__block.GetTable(targetCategory)
                colNames = list(myTable.GetColumnNames())
                for aTup in aList:
                    if aTup[1] in colNames:
                        cL = []
                        rList = list(myTable.GetColumn(cL, aTup[1]))
                        if len(rList) > 0:
                            # Cannot be unicode for comparisons
                            d[aTup[0]] = rList[0].encode("utf-8")
                        else:
                            d[aTup[0]] = None
            #
            #
            targetCategory = "pdbx_SG_project"
            if self.__block.IsTablePresent(targetCategory):
                aList = [("full_name_of_center", "full_name_of_center"), ("initial_of_center", "initial_of_center")]
                myTable = self.__block.GetTable(targetCategory)
                colNames = list(myTable.GetColumnNames())
                for aTup in aList:
                    if aTup[1] in colNames:
                        cL = []
                        rList = list(myTable.GetColumn(cL, aTup[1]))
                        if len(rList) > 0:
                            d[aTup[0]] = rList[0]
                        else:
                            d[aTup[0]] = None
            #
            targetCategory = "database_2"
            dbIdList = ["PDB", "BMRRB", "EMDB"]
            accessionD = {}
            for dbId in dbIdList:
                accessionD[dbId] = []
            #
            if self.__block.IsTablePresent(targetCategory):
                myTable = self.__block.GetTable(targetCategory)
                for dbId in dbIdList:
                    indexList = myTable.Search((dbId,), ("database_id",))
                    if len(indexList) > 0:
                        tList = []
                        for idx in range(0, len(indexList)):
                            tList.append(myTable(indexList[idx], "database_code"))
                        accessionD[dbId] = tList
                    else:
                        accessionD[dbId] = []

            d["accessions"] = accessionD

            targetCategory = "audit_author"
            if self.__block.IsTablePresent(targetCategory):
                targetAttribute = "name"
                myTable = self.__block.GetTable(targetCategory)
                colNames = list(myTable.GetColumnNames())
                cL = []
                rList = list(myTable.GetColumn(cL, targetAttribute))
                if len(rList) > 0:
                    d["audit_author"] = []
                    for r in rList:
                        d["audit_author"].append(r)
                else:
                    d["audit_author"] = []

            outObjD["dst"].setValue(d)
            return True
        except Exception as _e:  # noqa: F841
            if self._verbose:
                traceback.print_exc(file=self._lfh)
            return False



##
# File:    NmrUtils.py
# Date:    18-Oct-2013
#
# Updates:
# 12-Sep-2014 jdw    added formatCheckCsXyzOp() and atomNameCheckCsXyzOp()
#  4-Aug-2015 jdw    added atomNameCheckCsXyzAltOp() and uploadChemicalShiftAltOp()
##
"""
Module of NMR utilities.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import os
import sys
import traceback
import json
import re

from wwpdb.utils.wf.plugins.UtilsBase import UtilsBase
from wwpdb.utils.config.ConfigInfo import ConfigInfo

from wwpdb.utils.dp.RcsbDpUtility import RcsbDpUtility
from wwpdb.utils.dp.PdbxChemShiftReport import PdbxChemShiftReport

try:
    # We will have present on annotation system - but allow testing of DepUI merge without
    from wwpdb.apps.ann_tasks_v2.nmr.NmrChemShiftProcessUtils import NmrChemShiftProcessUtils
except ImportError:
    pass

try:
    # We will have present on annotation system - but allow testing of DepUI merge without
    from wwpdb.utils.nmr.NmrDpUtility import NmrDpUtility
except ImportError:
    pass


sys.stdout = sys.stderr


class NmrUtils(UtilsBase):

    """Utility class to perform NMR file format conversions.




    Each method in this class implements the method calling interface of the
    `ProcessRunner()` class.   This interface provides the keyword arguments:

    - inputObjectD   dictionary of input objects
    - outputObjectD  dictionary of output objects
    - userParameterD  dictionary of user adjustable parameters
    - internalParameterD dictionary of internal parameters

    Each method in the class handles its own exceptions and returns
    True on success or False otherwise.

    """

    def __init__(self, verbose=False, log=sys.stderr):
        super(NmrUtils, self).__init__(verbose, log)
        self.__cleanUp = True
        """Flag to remove any temporary directories created by this class.
        """
        #

    def ccpnExtractOp(self, **kwArgs):  # pylint: disable=unused-argument
        """Extract PDB and NMRSTAR files from CCPn project"""

        # Disabled as not python3 compatible
        if self._verbose:
            self._lfh.write("+NmrUtils.ccpnExtractOp() - DISABLED\n")
        return False

    def uploadChemicalShiftOp(self, **kwArgs):
        """Performs format check on a list of chemical shift files and concatenates these.

        Data sections are assigned to user provided input names corresponding to each input file.

        Output is a concatenated chemical shift file,  text status (ok,warning,error) and text list of
        warnings and/or errors.

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            csPathList = inpObjD["src1"].getValue()
            nameList = inpObjD["src2"].getValue()
            #
            csOutPath = outObjD["dst1"].getFilePathReference()
            dirPath = outObjD["dst1"].getDirPathReference()
            chkPath = outObjD["dst2"].getFilePathReference()

            logPath = os.path.join(dirPath, "upload-chemical-shift-check.log")
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            dp = RcsbDpUtility(tmpPath=dirPath, siteId=siteId, verbose=self._verbose, log=self._lfh)
            dp.setDebugMode(flag=True)

            dp.addInput(name="chemical_shifts_file_path_list", value=csPathList)
            dp.addInput(name="chemical_shifts_auth_file_name_list", value=nameList)
            dp.addInput(name="chemical_shifts_upload_check_file_path", value=chkPath)
            #
            dp.op("annot-chem-shifts-upload-check")
            dp.expLog(logPath)
            dp.exp(csOutPath)

            csr = PdbxChemShiftReport(inputPath=chkPath, verbose=self._verbose, log=self._lfh)
            status = str("".join(csr.getStatus())).lower()
            warnings = csr.getWarnings()
            errors = csr.getErrors()

            outObjD["dst3"].setValue(status)
            outObjD["dst4"].setValue(warnings)
            outObjD["dst5"].setValue(errors)

            #
            if self.__cleanUp:
                dp.cleanup()
            if self._verbose:
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftOp() - CS path list :    %s\n" % csPathList)
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftOp() - CS name list :    %s\n" % nameList)
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftOp() - CS output path :  %s\n" % csOutPath)
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftOp() Status code: %s\n" % status)
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftOp() Warning count : %d\n %s\n" % (len(warnings), ("\n").join(warnings)))
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftOp() Error count : %d\n %s\n" % (len(errors), ("\n").join(errors)))

            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    def uploadChemicalShiftAltOp(self, **kwArgs):
        """Performs format check on a list of chemical shift files and concatenates these.

        Data sections are assigned to user provided input names corresponding to each input file.

        Output is a concatenated chemical shift file, and check report file -

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            csPathList = []
            csPathListFilePath = inpObjD["src1"].getFilePathReference()
            nameList = []
            nameListFilePath = inpObjD["src2"].getFilePathReference()
            #
            ifh = open(csPathListFilePath, "r")
            for tline in ifh:
                txt = str(tline[:-1]).strip()
                csPathList.append(txt)
            ifh.close()
            #
            ifh = open(nameListFilePath, "r")
            for tline in ifh:
                txt = str(tline[:-1]).strip()
                nameList.append(txt)
            ifh.close()
            #
            csOutPath = outObjD["dst1"].getFilePathReference()
            dirPath = outObjD["dst1"].getDirPathReference()
            chkPath = outObjD["dst2"].getFilePathReference()

            logPath = os.path.join(dirPath, "upload-chemical-shift-check.log")
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            dp = RcsbDpUtility(tmpPath=dirPath, siteId=siteId, verbose=self._verbose, log=self._lfh)
            dp.setDebugMode(flag=True)

            dp.addInput(name="chemical_shifts_file_path_list", value=csPathList)
            dp.addInput(name="chemical_shifts_auth_file_name_list", value=nameList)
            dp.addInput(name="chemical_shifts_upload_check_file_path", value=chkPath)
            #
            dp.op("annot-chem-shifts-upload-check")
            dp.expLog(logPath)
            dp.exp(csOutPath)

            #
            if self.__cleanUp:
                dp.cleanup()
            if self._verbose:
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftAltOp() - CS path list :    %s\n" % csPathList)
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftAltOp() - CS name list :    %s\n" % nameList)
                self._lfh.write("+AnnotationUtils.uploadChemicalShiftAltOp() - CS output path :  %s\n" % csOutPath)
            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    def atomNameCheckCsXyzOp(self, **kwArgs):
        """Performs nomenclature and format check on input CS and XYZ files and returns an updated CS file, a CIF check report,
        status (text=ok,warning,error), and both  warnings and error messages as a list of strings.

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            csPath = inpObjD["src1"].getFilePathReference()
            xyzPath = inpObjD["src2"].getFilePathReference()

            csOutPath = outObjD["dst1"].getFilePathReference()
            chkPath = outObjD["dst2"].getFilePathReference()
            dirPath = outObjD["dst2"].getDirPathReference()
            logPath = os.path.join(dirPath, "atom-name-check-cs-xyz.log")
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            dp = RcsbDpUtility(tmpPath=dirPath, siteId=siteId, verbose=self._verbose, log=self._lfh)

            dp.imp(csPath)
            dp.addInput(name="coordinate_file_path", value=xyzPath)
            dp.addInput(name="chemical_shifts_coord_check_file_path", value=chkPath)

            dp.op("annot-chem-shifts-atom-name-check")

            dp.expLog(logPath)
            dp.exp(csOutPath)

            csr = PdbxChemShiftReport(inputPath=chkPath, verbose=self._verbose, log=self._lfh)
            status = str("".join(csr.getStatus())).lower()
            warnings = csr.getWarnings()
            errors = csr.getErrors()

            outObjD["dst3"].setValue(status)
            outObjD["dst4"].setValue(warnings)
            outObjD["dst5"].setValue(errors)

            if self.__cleanUp:
                dp.cleanup()
            if self._verbose:
                self._lfh.write("+AnnotationUtils.atomNameCheckCsXyzOp() - Cs input  file path:    %s\n" % csPath)
                self._lfh.write("+AnnotationUtils.atomNameCheckCsXyzOp() - Report file path:       %s\n" % chkPath)
                self._lfh.write("+AnnotationUtils.atomNameShiftOp() Status code: %s\n" % status)
                self._lfh.write("+AnnotationUtils.atomNameShiftOp() Warning count : %d\n %s\n" % (len(warnings), ("\n").join(warnings)))
                self._lfh.write("+AnnotationUtils.atomNameShiftOp() Error count : %d\n %s\n" % (len(errors), ("\n").join(errors)))

            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    def atomNameCheckCsXyzAltOp(self, **kwArgs):
        """Performs nomenclature and format check on input CS and XYZ files and returns an updated CS file, a CIF check report.

        * workflow version * returns chemical shift file and check report output --
        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            csPath = inpObjD["src1"].getFilePathReference()
            xyzPath = inpObjD["src2"].getFilePathReference()

            csOutPath = outObjD["dst1"].getFilePathReference()
            chkPath = outObjD["dst2"].getFilePathReference()
            dirPath = outObjD["dst2"].getDirPathReference()
            logPath = os.path.join(dirPath, "atom-name-check-cs-xyz.log")
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            dp = RcsbDpUtility(tmpPath=dirPath, siteId=siteId, verbose=self._verbose, log=self._lfh)

            dp.imp(csPath)
            dp.addInput(name="coordinate_file_path", value=xyzPath)
            dp.addInput(name="chemical_shifts_coord_check_file_path", value=chkPath)

            dp.op("annot-chem-shifts-atom-name-check")

            dp.expLog(logPath)
            dp.exp(csOutPath)

            if self.__cleanUp:
                dp.cleanup()
            if self._verbose:
                self._lfh.write("+AnnotationUtils.atomNameCheckCsXyzAltOp() - Cs input  file path:    %s\n" % csPath)
                self._lfh.write("+AnnotationUtils.atomNameCheckCsXyzAltOp() - Report file path:       %s\n" % chkPath)
            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    def autoChemShiftProcessOp(self, **kwArgs):
        """Performs chemical shift file update & nomenclature and format check on input CS and XYZ files and returns an updated CS file, a CIF check report.

        * workflow version * returns chemical shift file and check report output --
        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            xyzPath = inpObjD["src1"].getFilePathReference()
            csPath = inpObjD["src2"].getFilePathReference()
            #
            xyzOutPath = outObjD["dst1"].getFilePathReference()
            csOutPath = outObjD["dst2"].getFilePathReference()
            dirPath = outObjD["dst2"].getDirPathReference()
            validationReportPath = outObjD["dst3"].getFilePathReference()
            xmlReportPath = outObjD["dst4"].getFilePathReference()
            validationFullReportPath = outObjD["dst5"].getFilePathReference()
            pngReportPath = outObjD["dst6"].getFilePathReference()
            svgReportPath = outObjD["dst7"].getFilePathReference()
            csReportPath = outObjD["dst8"].getFilePathReference()
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            util = NmrChemShiftProcessUtils(siteId=siteId, verbose=self._verbose, log=self._lfh)
            util.setWorkingDirPath(dirPath=dirPath)
            util.setInputModelFileName(fileName=xyzPath)
            util.setInputCsFileName(fileName=csPath)
            util.setOutputModelFileName(fileName=xyzOutPath)
            util.setOutputCsFileName(fileName=csOutPath)
            util.setOutputReportFileName(fileName=csReportPath)
            util.setOutputValidationFileList(dstPathList=[validationReportPath, xmlReportPath, validationFullReportPath, pngReportPath, svgReportPath])
            util.run()
            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    # DepUI for NMR unified data: NEF consistency check with model
    #   action: nmr-nef-consistency-check
    #   src0.content: nmr-data-config,      src0.format: json
    #   src1.content: nmr-data-nef,         src1.format: nmr-star
    #   src2.content: model,                src2.format: pdbx
    #   prc2.content: model (deposit),      prc2.format: pdbx
    #   dst.content:  nmr-data-nef-report,  dst.format:  json
    def nefConsistencyCheckOp(self, **kwArgs):
        """Performs consistency check on input NEF with coordinate and outputs a JSON report file, which provides diagnostic information to depositor.

        Returns True for success or False for warnings/errors.

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            cnfInpPath = inpObjD["src0"].getFilePathReference()
            nefInpPath = inpObjD["src1"].getFilePathReference()
            cifInpPath = inpObjD["src2"].getFilePathReference()
            prcInpPath = inpObjD["prc2"].getFilePathReference()
            logOutPath = outObjD["dst"].getFilePathReference()
            #
            dp = NmrDpUtility(verbose=self._verbose, log=self._lfh)
            dp.setVerbose(True)
            dp.setSource(nefInpPath)
            dp.addInput(name="coordinate_file_path", value=cifInpPath, type="file")
            dp.addInput(name="proc_coord_file_path", value=prcInpPath, type="file")

            if os.path.exists(cnfInpPath):

                with open(cnfInpPath, "r") as file:
                    conf = json.loads(file.read())

                for item in conf.keys():
                    dp.addInput(name=item, value=conf[item], type="param")

            dp.setLog(logOutPath)
            stat = dp.op("nmr-nef-consistency-check")
            #
            if self._verbose:
                self._lfh.write("+NmrUtils.nefConsistencyCheckOp() - NEF input file path:      %s\n" % nefInpPath)
                self._lfh.write("+NmrUtils.nefConsistencyCheckOp() - mmCIF input file path:    %s\n" % cifInpPath)
                self._lfh.write("+NmrUtils.nefConsistencyCheckOp() - JSON output file path:    %s\n" % logOutPath)
            return stat
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    # DepUI for NMR unified data: NMR-STAR V3.2 consistency check with model
    #   action: nmr-str-consistency-check
    #   src0.content: nmr-data-config,      src0.format: json
    #   src1.content: nmr-data-str,         src1.format: nmr-star
    #   src2.content: model,                src2.format: pdbx
    #   prc2.content: model (deposit),      prc2.format: pdbx
    #   dst.content:  nmr-data-str-report,  dst.format:  json
    def strConsistencyCheckOp(self, **kwArgs):
        """Performs consistency check on input NMR-STAR V3.2 with coordinate and outputs a JSON report file, which provides diagnostic information to depositor.

        Returns True for success or False for warnings/errors.

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            cnfInpPath = inpObjD["src0"].getFilePathReference()
            strInpPath = inpObjD["src1"].getFilePathReference()
            cifInpPath = inpObjD["src2"].getFilePathReference()
            prcInpPath = inpObjD["prc2"].getFilePathReference()
            logOutPath = outObjD["dst"].getFilePathReference()
            #
            dp = NmrDpUtility(verbose=self._verbose, log=self._lfh)
            dp.setVerbose(True)
            dp.setSource(strInpPath)
            dp.addInput(name="coordinate_file_path", value=cifInpPath, type="file")
            dp.addInput(name="proc_coord_file_path", value=prcInpPath, type="file")

            if os.path.exists(cnfInpPath):

                with open(cnfInpPath, "r") as file:
                    conf = json.loads(file.read())

                for item in conf.keys():
                    dp.addInput(name=item, value=conf[item], type="param")

            dp.setLog(logOutPath)
            stat = dp.op("nmr-str-consistency-check")
            #
            if self._verbose:
                self._lfh.write("+NmrUtils.strConsistencyCheckOp() - NMR-STAR V3.2 input file path:    %s\n" % strInpPath)
                self._lfh.write("+NmrUtils.strConsistencyCheckOp() - mmCIF input file path:            %s\n" % cifInpPath)
                self._lfh.write("+NmrUtils.strConsistencyCheckOp() - JSON output file path:            %s\n" % logOutPath)
            return stat
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    # DepUI for NMR legacy data: Chemical shifts/restraints file check with model
    #   action: nmr-cs-str-check
    #   src1.content: nmr-cs-path-list,     src1.format: string
    #   src2.content: nmr-mr-path-list,     src2.format: string
    #   src3.content: model,                src3.format: pdbx
    #   prc3.content: model (deposit),      prc3.format: pdbx
    #   dst.content:  nmr-data-str-report,  dst.format:  json
    def csStrConsistencyCheckOp(self, **kwArgs):
        """Performs consistency check on input chemical shift/restraint list with coordinate and outputs a JSON report file, which provides diagnostic information to depositor.

        Returns True for success or False for warnings/errors.

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            csPathList = []
            csPathListFilePath = inpObjD["src1"].getFilePathReference()
            #
            ifh = open(csPathListFilePath, "r")
            for tline in ifh:
                txt = str(tline[:-1]).strip()
                csPathList.append(txt)
            ifh.close()
            #
            mrPathList = []
            arPathList = []
            mrInpPath = inpObjD["src2"].getFilePathReference()
            #
            if os.path.exists(mrInpPath):

                with open(mrInpPath, "r") as file:
                    mr_list = json.loads(file.read())

                datablock_pattern = re.compile(r"\s*data_\S+\s*")
                sf_anonymous_pattern = re.compile(r"\s*save_\S+\s*")
                save_pattern = re.compile(r"\s*save_\s*")
                loop_pattern = re.compile(r"\s*loop_\s*")
                stop_pattern = re.compile(r"\s*stop_\s*")

                for mr in mr_list:
                    mr_file = mr["file_name"]
                    mr_orig_file = mr["original_file_name"]
                    mr_file_type = mr["file_type"]

                    # mr_orig_file_ext = os.path.splitext(mr_orig_file)[1]
                    # if (mr_orig_file_ext == '.str' or mr_orig_file_ext == '.nef') and mr_file_type == 'nm-res-oth':

                    if mr_file_type.startswith("nm-res") or mr_file_type.startswith("nm-aux"):
                        has_datablock = False
                        has_anonymous_saveframe = False
                        has_save = False
                        has_loop = False
                        has_stop = False

                        with open(mr_file, "r") as ifp:
                            for line in ifp:
                                if datablock_pattern.match(line):
                                    has_datablock = True
                                elif sf_anonymous_pattern.match(line):
                                    has_anonymous_saveframe = True
                                elif save_pattern.match(line):
                                    has_save = True
                                elif loop_pattern.match(line):
                                    has_loop = True
                                elif stop_pattern.match(line):
                                    has_stop = True

                            ifp.close()

                        if has_datablock or has_anonymous_saveframe or has_save or has_loop or has_stop:  # NMR-STAR or NEF (DAOTHER-6830)
                            mrPathList.append(mr_file)
                        else:
                            arPathList.append({"file_name": mr_file, "file_type": mr_file_type, "original_file_name": mr_orig_file})
            #
            cifInpPath = inpObjD["src3"].getFilePathReference()
            prcInpPath = inpObjD["prc3"].getFilePathReference()
            logOutPath = outObjD["dst"].getFilePathReference()
            #
            dp = NmrDpUtility(verbose=self._verbose, log=self._lfh)
            dp.setVerbose(True)
            dp.addInput(name="chem_shift_file_path_list", value=csPathList, type="file_list")
            if len(mrPathList) > 0:
                dp.addInput(name="restraint_file_path_list", value=mrPathList, type="file_list")
            if len(arPathList) > 0:
                dp.addInput(name="atypical_restraint_file_path_list", value=arPathList, type="file_dict_list")
            dp.addInput(name="coordinate_file_path", value=cifInpPath, type="file")
            dp.addInput(name="proc_coord_file_path", value=prcInpPath, type="file")

            dp.addInput(name="nonblk_anomalous_cs", value=True, type="param")
            dp.addInput(name="nonblk_bad_nterm", value=True, type="param")
            dp.addInput(name="resolve_conflict", value=True, type="param")
            dp.addInput(name="check_mandatory_tag", value=False, type="param")

            dp.setLog(logOutPath)
            stat = dp.op("nmr-cs-str-consistency-check")
            #
            if self._verbose:
                self._lfh.write("+NmrUtils.csStrConsistencyCheckOp() - CS file path list:        %s\n" % csPathList)
                if len(mrPathList) > 0:
                    self._lfh.write("+NmrUtils.csStrConsistencyCheckOp() - MR file path list:        %s\n" % mrPathList)
                if len(arPathList) > 0:
                    self._lfh.write("+NmrUtils.csStrConsistencyCheckOp() - AR file path list:        %s\n" % arPathList)
                self._lfh.write("+NmrUtils.csStrConsistencyCheckOp() - mmCIF input file path:    %s\n" % cifInpPath)
                self._lfh.write("+NmrUtils.csStrConsistencyCheckOp() - JSON output file path:    %s\n" % logOutPath)
            return stat
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    # DepUI for NMR unified data: NEF -> NMR-STAR V3.2 conversion and deposition
    #   action: nmr-nef2str-deposit
    #   src0.content: nmr-data-config,      src0.format: json
    #   src1.content: nmr-data-nef,         src1.format: nmr-star
    #   src2.content: model,                src2.format: pdbx
    #   prc2.content: model (deposit),      prc2.format: pdbx
    #   src3.content: nmr-data-nef-report,  src3.format: json
    #   dst1.content: nmr-data-nef,         dst1.format: nmr-star
    #   dst2.content: nmr-data-str,         dst2.format: nmr-star
    #   dst3.content: nmr-data-nef-report,  dst3.format: json
    #   dst4.content: nmr-data-str-report,  dst4.format: json
    def nef2strDepositOp(self, **kwArgs):
        """Perform NEF to NMR-STAR V3.2 format conversion operation (special processing for deposition sessions)

        Returns True for success or False for warnings/errors.

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            cnfInpPath = inpObjD["src0"].getFilePathReference()
            nefInpPath = inpObjD["src1"].getFilePathReference()
            cifInpPath = inpObjD["src2"].getFilePathReference()
            prcInpPath = inpObjD["prc2"].getFilePathReference()
            logInpPath = inpObjD["src3"].getFilePathReference()
            nefOutPath = outObjD["dst1"].getFilePathReference()
            strOutPath = outObjD["dst2"].getFilePathReference()
            logOutPath1 = outObjD["dst3"].getFilePathReference()
            logOutPath2 = outObjD["dst4"].getFilePathReference()
            #
            dp = NmrDpUtility(verbose=self._verbose, log=self._lfh)
            dp.setVerbose(True)
            dp.setSource(nefInpPath)
            dp.addInput(name="coordinate_file_path", value=cifInpPath, type="file")
            dp.addInput(name="proc_coord_file_path", value=prcInpPath, type="file")
            dp.addInput(name="report_file_path", value=logInpPath, type="file")

            if os.path.exists(cnfInpPath):

                with open(cnfInpPath, "r") as file:
                    conf = json.loads(file.read())

                for item in conf.keys():
                    dp.addInput(name=item, value=conf[item], type="param")

            dp.setDestination(nefOutPath)
            dp.addOutput(name="nmr-star_file_path", value=strOutPath, type="file")
            dp.addOutput(name="report_file_path", value=logOutPath2, type="file")
            dp.addOutput(name="insert_entry_id_to_loops", value=True, type="param")
            dp.addOutput(name="leave_intl_note", value=False, type="param")
            dp.setLog(logOutPath1)
            stat = dp.op("nmr-nef2str-deposit")
            #
            if self._verbose:
                self._lfh.write("+NmrUtils.nef2strDepositOp() - NEF input file path:        %s\n" % nefInpPath)
                self._lfh.write("+NmrUtils.nef2strDepositOp() - mmCIF input file path:      %s\n" % cifInpPath)
                self._lfh.write("+NmrUtils.nef2strDepositOp() - JSON input file path:       %s\n" % logInpPath)
                self._lfh.write("+NmrUtils.nef2strDepositOp() - NEF output file path:       %s\n" % nefOutPath)
                self._lfh.write("+NmrUtils.nef2strDepositOp() - NMR-STAR V3.2 file path:    %s\n" % strOutPath)
                self._lfh.write("+NmrUtils.nef2strDepositOp() - JSON output file path 1:    %s\n" % logOutPath1)
                self._lfh.write("+NmrUtils.nef2strDepositOp() - JSON output file path 2:    %s\n" % logOutPath2)
            return stat
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    # DepUI for NMR unified data: NMR-STAR V3.2 conversion and deposition
    #   action: nmr-str2str-deposit
    #   src0.content: nmr-data-config,      src0.format: json
    #   src1.content: nmr-data-str,         src1.format: nmr-star
    #   src2.content: model,                src2.format: pdbx
    #   prc2.content: model (deposit),      prc2.format: pdbx
    #   src3.content: nmr-data-str-report,  src3.format: json
    #   dst1.content: nmr-data-str,         dst1.format: nmr-star
    #   dst2.content: nmr-data-str-report,  dst2.format: json
    def str2strDepositOp(self, **kwArgs):
        """Perform NMR-STAR V3.2 format conversion operation (special processing for deposition sessions)

        Returns True for success or False for warnings/errors.

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            cnfInpPath = inpObjD["src0"].getFilePathReference()
            strInpPath = inpObjD["src1"].getFilePathReference()
            cifInpPath = inpObjD["src2"].getFilePathReference()
            prcInpPath = inpObjD["prc2"].getFilePathReference()
            logInpPath = inpObjD["src3"].getFilePathReference()
            strOutPath = outObjD["dst1"].getFilePathReference()
            logOutPath = outObjD["dst2"].getFilePathReference()
            #
            dp = NmrDpUtility(verbose=self._verbose, log=self._lfh)
            dp.setVerbose(True)
            dp.setSource(strInpPath)
            dp.addInput(name="coordinate_file_path", value=cifInpPath, type="file")
            dp.addInput(name="proc_coord_file_path", value=prcInpPath, type="file")
            dp.addInput(name="report_file_path", value=logInpPath, type="file")

            if os.path.exists(cnfInpPath):

                with open(cnfInpPath, "r") as file:
                    conf = json.loads(file.read())

                for item in conf.keys():
                    dp.addInput(name=item, value=conf[item], type="param")

            dp.setDestination(strOutPath)
            dp.addOutput(name="insert_entry_id_to_loops", value=True, type="param")
            dp.addOutput(name="leave_intl_note", value=False, type="param")
            dp.setLog(logOutPath)
            stat = dp.op("nmr-str2str-deposit")
            #
            if self._verbose:
                self._lfh.write("+NmrUtils.str2strDepositOp() - NMR-STAR V3.2 input file path:     %s\n" % strInpPath)
                self._lfh.write("+NmrUtils.str2strDepositOp() - mmCIF input file path:             %s\n" % cifInpPath)
                self._lfh.write("+NmrUtils.str2strDepositOp() - JSON input file path:              %s\n" % logInpPath)
                self._lfh.write("+NmrUtils.str2strDepositOp() - NMR-STAR V3.2 output file path:    %s\n" % strOutPath)
                self._lfh.write("+NmrUtils.str2strDepositOp() - JSON output file path:             %s\n" % logOutPath)
            return stat
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    # NMR-STAR V3.2 -> NEF conversion and release
    #   action: nmr-str2nef-deposit
    #   src1.content: nmr-data-str,         src1.format: nmr-star
    #   src2.content: model,                src2.format: pdbx
    #   dst0.content: nmr-data-str-report,  dst0.format: json
    #   dst1.content: nmr-data-str,         dst1.format: nmr-star
    #   dst2.content: nmr-data-nef,         dst2.format: nmr-star
    #   dst3.content: nmr-data-str-report,  dst3.format: json
    #   dst4.content: nmr-data-nef-report,  dst4.format: json
    def str2nefReleaseOp(self, **kwArgs):
        """Perform NMR-STAR V3.2 to NEF format conversion operation for public release

        Returns True for success or False for warnings/errors.

        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            strInpPath = inpObjD["src1"].getFilePathReference()
            cifInpPath = inpObjD["src2"].getFilePathReference()
            logOutPath = outObjD["dst0"].getFilePathReference()
            strOutPath = outObjD["dst1"].getFilePathReference()
            nefOutPath = outObjD["dst2"].getFilePathReference()
            logOutPath1 = outObjD["dst3"].getFilePathReference()
            logOutPath2 = outObjD["dst4"].getFilePathReference()
            #
            dp = NmrDpUtility(verbose=self._verbose, log=self._lfh)
            dp.setVerbose(True)
            dp.setSource(strInpPath)
            dp.addInput(name="coordinate_file_path", value=cifInpPath, type="file")
            dp.addInput(name="nonblk_anomalous_cs", value=True, type="param")
            dp.addInput(name="nonblk_bad_nterm", value=True, type="param")
            dp.addInput(name="resolve_conflict", value=True, type="param")
            dp.addInput(name="check_mandatory_tag", value=True, type="param")
            dp.setLog(logOutPath)
            stat = dp.op("nmr-str-consistency-check")
            #
            dp.setSource(strInpPath)
            dp.addInput(name="coordinate_file_path", value=cifInpPath, type="file")
            dp.addInput(name="report_file_path", value=logOutPath, type="file")
            dp.setDestination(strOutPath)
            dp.addOutput(name="nef_file_path", value=nefOutPath, type="file")
            dp.addOutput(name="report_file_path", value=logOutPath2, type="file")
            dp.addOutput(name="insert_entry_id_to_loops", value=True, type="param")
            dp.addOutput(name="leave_intl_note", value=False, type="param")
            dp.setLog(logOutPath1)
            stat = dp.op("nmr-str2nef-release")
            #
            if self._verbose:
                self._lfh.write("+NmrUtils.str2nefReleaseOp() - NMR-STAR V3.2 input file path:     %s\n" % strInpPath)
                self._lfh.write("+NmrUtils.str2nefReleaseOp() - mmCIF input file path:             %s\n" % cifInpPath)
                self._lfh.write("+NmrUtils.str2nefReleaseOp() - JSON input file path:              %s\n" % logOutPath)
                self._lfh.write("+NmrUtils.str2nefReleaseOp() - NMR-STAR V3.2 output file path:    %s\n" % strOutPath)
                self._lfh.write("+NmrUtils.str2nefReleaseOp() - NEF file path:                     %s\n" % nefOutPath)
                self._lfh.write("+NmrUtils.str2nefReleaseOp() - JSON output file path 1:           %s\n" % logOutPath1)
                self._lfh.write("+NmrUtils.str2nefReleaseOp() - JSON output file path 2:           %s\n" % logOutPath2)
            return stat
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False

    def autoNmrNefProcessOp(self, **kwArgs):
        """Performs chemical shift file update & nomenclature and format check on input CS and XYZ files and returns an updated CS file, a CIF check report.

        * workflow version * returns chemical shift file and check report output --
        """
        try:
            (inpObjD, outObjD, _uD, _pD) = self._getArgs(kwArgs)
            xyzPath = inpObjD["src1"].getFilePathReference()
            nefPath = inpObjD["src2"].getFilePathReference()
            #
            xyzOutPath = outObjD["dst1"].getFilePathReference()
            nefOutPath = outObjD["dst2"].getFilePathReference()
            dirPath = outObjD["dst2"].getDirPathReference()
            #
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            util = NmrChemShiftProcessUtils(siteId=siteId, verbose=self._verbose, log=self._lfh)
            util.setWorkingDirPath(dirPath=dirPath)
            util.setInputModelFileName(fileName=xyzPath)
            util.setInputNefFileName(fileName=nefPath)
            util.setOutputModelFileName(fileName=xyzOutPath)
            util.setOutputNefFileName(fileName=nefOutPath)
            util.runNefProcess()
            return True
        except Exception as _e:  # noqa: F841
            traceback.print_exc(file=self._lfh)
            return False
